require "import"
import "android.content.Context"
import "android.content.Intent"
import "java.util.Locale"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "android.speech.RecognizerIntent"

function destroyRecognizer(recognizer)
  if recognizer then
    recognizer.destroy()
    recognizer = nil
  end
  return true
end

function startSpeechRecognition()
  local speechRecognizer = SpeechRecognizer.createSpeechRecognizer(this)

  local recognitionListener = RecognitionListener {
    onResults = function(results)
      local resultArray = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
      if resultArray ~= nil and resultArray.size() > 0 then
        local recognizedText = resultArray.get(0) -- Original recognized text

        -- Perform word replacement and text transformations
        local replacedText = replaceWords(recognizedText)

        -- Add the ending symbol ("।") at the end of the text
        replacedText = replacedText .. "।"

        local editText = service.getEditText()
        service.insertText(editText, replacedText .. " ") -- Insert the text obtained after replacement
        service.speak(replacedText)
      end
      destroyRecognizer(speechRecognizer)
    end,
    onError = function()
      destroyRecognizer(speechRecognizer)
      return false
    end
  }

  local recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
  recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
  recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "pa-IN") -- Set recognition language to Punjabi
  recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, this.getPackageName())
  speechRecognizer.startListening(recognizerIntent)
  speechRecognizer.setRecognitionListener(recognitionListener)

  return true
end

function replaceWords(text)
  local wordReplacements = {
    ["comma"] = ",",
    ["period"] = "।",
    ["question mark"] = "?",
    ["exclamation mark"] = "!",
    ["ending symbol"] = "।" -- Add the "ending symbol" applicable in Punjabi
  }

  for originalWord, replacementWord in pairs(wordReplacements) do
    text = text:gsub(originalWord, replacementWord)
  end

  return text
end

startSpeechRecognition()
